function z=astm(op,varargin)
%ASTM Database of ASTM E308 table 5 and 6 color weighting functions.
%   Z=ASTM('CWF',ILLOBS,WL,COMP), where ILLOBS is an illuminant/observer
%   specification, WL a wavelength range and COMP either 'compensated' or
%   'uncompensated', returns the corresponding color weighting function
%   with size [length(WL) 3]. If either end of the specified wavelength
%   range is falls inside the tabulated range, the cwf is summed up from
%   respective end, as recommended by ASTM. If either end of the specified
%   wavelength range is falls outside the tabulated, the cwf is padded with
%   zeros. If the illumination/observer isn't tabulated by ASTM or if the
%   specified wavelength interval does not coincide with the tabulated
%   interval, the return value is empty.
%
%   The illuminant/observer specification, ILLOBS is a char array, e.g.
%   'D50/2'. If omitted or empty, it is replaced by the default
%   illuminant/observer.
%
%   The wavelength range, WL, is a vector of wavelengths. If omitted or
%   empty, it is replaced by the default wavelength range. Note that the
%   returned wavelength range is NOT the same as the input wavelength
%   range. The input wavelength range is only used to select a suitable
%   table.
%
%   COMP can only have two settings, 'compensated' or 'uncompensated'. The
%   cwf returned for 'compensated' is suitable for reflectance spectra that
%   already have been spectrally bandpass compensated, i.e. ASTM E308 Table
%   5. When COMP is 'uncompensated', the corresponding ASTM E308 Table 6 is
%   returned. If COMP is not specified or empty, it is replaced by the
%   default spectral type.
%
%   Example:
%      % Compare the default color matching function with cwf of D75/2
%      def=astm('cwf','');
%      d75=astm('cwf','D75/2');
%      plot(def.wl,def.weights,d75.wl,d75.weights)
%
%   See also: MAKECWF,ROO2XYZ, OPTGETPREF, DWL

% Part of the OptProp toolbox, $Version: 2.1 $
% Author:  Jerker Wgberg, More Research & DPC, Sweden
% Email: ['jerker.wagberg' char(64) 'more.se']

% $Id: astm.m 23 2007-01-28 22:55:34Z jerkerw $

	persistent cwf lam

	if isempty(cwf)
		lam{1} = 360:10:780;
		lam{2} = 400:20:700;

		cwf.A_2.whitepoint = [109.850 100.000 35.585];
		cwf.A_2.wl = lam(1);
		cwf.A_2.compensated{1} = [
			 0.0000      0.0000      0.0000
			 0.0000      0.0000      0.0010
			 0.0010      0.0000      0.0050
			 0.0050      0.0000      0.0210
			 0.0170      0.0000      0.0830
			 0.0700      0.0020      0.3330
			 0.2720      0.0080      1.3090
			 0.6440      0.0270      3.1440
			 0.9240      0.0610      4.6350
			 1.0360      0.1170      5.4610
			 1.0170      0.2090      5.8380
			 0.7790      0.3620      5.1280
			 0.4280      0.6180      3.6390
			 0.1600      1.0390      2.3320
			 0.0240      1.8020      1.5130
			 0.0590      3.0910      0.9620
			 0.4280      4.7560      0.5330
			 1.2100      6.3200      0.3050
			 2.3130      7.5990      0.1620
			 3.7350      8.5710      0.0750
			 5.5110      9.2190      0.0360
			 7.5730      9.4560      0.0210
			 9.7180      9.2240      0.0170
			11.5830      8.5430      0.0130
			12.7060      7.5470      0.0100
			12.6710      6.3600      0.0050
			11.3470      5.0610      0.0020
			 9.0100      3.7160      0.0010
			 6.5510      2.5590      0.0000
			 4.3450      1.6390      0.0000
			 2.6260      0.9710      0.0000
			 1.4570      0.5330      0.0000
			 0.7940      0.2890      0.0000
			 0.4060      0.1470      0.0000
			 0.2070      0.0750      0.0000
			 0.1090      0.0390      0.0000
			 0.0560      0.0200      0.0000
			 0.0290      0.0100      0.0000
			 0.0140      0.0050      0.0000
			 0.0070      0.0030      0.0000
			 0.0040      0.0010      0.0000
			 0.0020      0.0010      0.0000
			 0.0010      0.0000      0.0000
			];
		cwf.A_2.uncompensated{1} = [
			  0.0000	  0.0000	  0.0000
			  0.0000	  0.0000	  0.0010
			  0.0010	  0.0000	  0.0050
			  0.0040	  0.0000	  0.0180
			  0.0170	  0.0000	  0.0810
			  0.0570	  0.0020	  0.2720
			  0.2460	  0.0070	  1.1780
			  0.6600	  0.0250	  3.2140
			  0.9420	  0.0590	  4.7100
			  1.0390	  0.1130	  5.4540
			  1.0430	  0.2050	  5.9690
			  0.7900	  0.3530	  5.2090
			  0.4160	  0.6080	  3.6020
			  0.1480	  1.0120	  2.2770
			  0.0160	  1.7490	  1.4930
			  0.0280	  3.0470	  0.9630
			  0.3880	  4.7780	  0.5050
			  1.1870	  6.3450	  0.3050
			  2.2880	  7.6250	  0.1570
			  3.7020	  8.5940	  0.0710
			  5.4840	  9.2550	  0.0340
			  7.5620	  9.4960	  0.0200
			  9.7390	  9.2650	  0.0180
			 11.6440	  8.5670	  0.0130
			 12.8110	  7.5630	  0.0100
			 12.7820	  6.3650	  0.0040
			 11.4600	  5.0760	  0.0020
			  8.9910	  3.6890	  0.0010
			  6.5360	  2.5430	  0.0000
			  4.2960	  1.6160	  0.0000
			  2.5830	  0.9540	  0.0000
			  1.4050	  0.5140	  0.0000
			  0.7800	  0.2830	  0.0000
			  0.3880	  0.1400	  0.0000
			  0.2000	  0.0720	  0.0000
			  0.1060	  0.0380	  0.0000
			  0.0540	  0.0200	  0.0000
			  0.0280	  0.0100	  0.0000
			  0.0140	  0.0050	  0.0000
			  0.0070	  0.0020	  0.0000
			  0.0030	  0.0010	  0.0000
			  0.0020	  0.0010	  0.0000
			  0.0010	  0.0000	  0.0000
			];

		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  A/10

		cwf.A_10.whitepoint = [111.144 100.000 35.200];
		cwf.A_10.wl = lam(1);
		cwf.A_10.compensated{1} = [
			 0.0000      0.0000      0.0000
			 0.0000      0.0000      0.0000
			 0.0000      0.0000      0.0000
			 0.0020      0.0000      0.0080
			 0.0250      0.0030      0.1100
			 0.1340      0.0140      0.6150
			 0.3770      0.0390      1.7920
			 0.6860      0.0840      3.3860
			 0.9640      0.1560      4.9440
			 1.0800      0.2590      5.8060
			 1.0060      0.4240      5.8120
			 0.7310      0.6960      4.9190
			 0.3430      1.0820      3.3000
			 0.0780      1.6160      1.9730
			 0.0220      2.4220      1.1520
			 0.2180      3.5290      0.6580
			 0.7500      4.8400      0.3820
			 1.6420      6.1000      0.2110
			 2.8420      7.2500      0.1020
			 4.3360      8.1140      0.0320
			 6.2000      8.7580      0.0010
			 8.2620      8.9880      0.0000
			10.2270      8.7600      0.0000
			11.9450      8.3040      0.0000
			12.7460      7.4680      0.0000
			12.3370      6.3230      0.0000
			10.8170      5.0330      0.0000
			 8.5600      3.7440      0.0000
			 6.0140      2.5060      0.0000
			 3.8870      1.5600      0.0000
			 2.3090      0.9110      0.0000
			 1.2760      0.4990      0.0000
			 0.6660      0.2590      0.0000
			 0.3360      0.1300      0.0000
			 0.1660      0.0650      0.0000
			 0.0820      0.0320      0.0000
			 0.0400      0.0160      0.0000
			 0.0200      0.0080      0.0000
			 0.0100      0.0040      0.0000
			 0.0050      0.0020      0.0000
			 0.0030      0.0010      0.0000
			 0.0010      0.0010      0.0000
			 0.0010      0.0000      0.0000
			];
		cwf.A_10.uncompensated{1} = [
			  0.0000	  0.0000	  0.0000
			  0.0000	  0.0000	  0.0000
			  0.0000	  0.0000	  0.0000
			  0.0020	  0.0000	  0.0070
			  0.0180	  0.0020	  0.0780
			  0.1180	  0.0120	  0.5400
			  0.3720	  0.0380	  1.7600
			  0.6860	  0.0820	  3.3740
			  0.9820	  0.1540	  5.0240
			  1.0940	  0.2550	  5.8760
			  1.0240	  0.4140	  5.8820
			  0.7470	  0.6880	  5.0230
			  0.3260	  1.0730	  3.2360
			  0.0610	  1.5890	  1.9260
			  0.0030	  2.3970	  1.1290
			  0.1890	  3.5030	  0.6380
			  0.7170	  4.8570	  0.3770
			  1.6170	  6.0960	  0.2050
			  2.8230	  7.2900	  0.1000
			  4.2960	  8.1160	  0.0280
			  6.1770	  8.7990	 -0.0030
			  8.2850	  9.0390	  0.0010
			 10.2180	  8.7580	  0.0000
			 12.0410	  8.3500	  0.0000
			 12.8500	  7.4920	  0.0000
			 12.4410	  6.3370	  0.0000
			 10.8720	  5.0250	  0.0000
			  8.6040	  3.7530	  0.0000
			  5.9510	  2.4690	  0.0000
			  3.8460	  1.5370	  0.0000
			  2.2590	  0.8910	  0.0000
			  1.2420	  0.4850	  0.0000
			  0.6430	  0.2500	  0.0000
			  0.3240	  0.1260	  0.0000
			  0.1600	  0.0620	  0.0000
			  0.0780	  0.0300	  0.0000
			  0.0390	  0.0150	  0.0000
			  0.0190	  0.0070	  0.0000
			  0.0100	  0.0040	  0.0000
			  0.0050	  0.0020	  0.0000
			  0.0020	  0.0010	  0.0000
			  0.0010	  0.0010	  0.0000
			  0.0010	  0.0000	  0.0000
			];

		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  C/2

		cwf.C_2.whitepoint = [98.074 100.000 118.232];
		cwf.C_2.wl = lam([1 2]);
		cwf.C_2.compensated{1} = [
			0.0000      0.0000      0.0000
			0.0010      0.0000      0.0040
			0.0040      0.0000      0.0170
			0.0180      0.0010      0.0840
			0.0760      0.0020      0.3580
			0.3250      0.0090      1.5470
			1.2920      0.0380      6.2070
			2.9680      0.1230     14.4960
			3.9590      0.2610     19.8600
			3.9310      0.4430     20.7280
			3.3600      0.6920     19.2860
			2.2830      1.0610     15.0220
			1.1160      1.6120      9.4790
			0.3630      2.3580      5.2860
			0.0480      3.4140      2.8680
			0.0920      4.8420      1.5120
			0.5780      6.4490      0.7200
			1.5190      7.9360      0.3810
			2.7860      9.1450      0.1950
			4.2850      9.8310      0.0860
			5.8770      9.8340      0.0380
			7.3230      9.1480      0.0200
			8.4140      7.9900      0.0150
			8.9850      6.6290      0.0100
			8.9580      5.3210      0.0070
			8.3240      4.1770      0.0030
			7.0550      3.1460      0.0010
			5.3270      2.1960      0.0000
			3.6920      1.4420      0.0000
			2.3520      0.8870      0.0000
			1.3600      0.5030      0.0000
			0.7130      0.2610      0.0000
			0.3640      0.1320      0.0000
			0.1720      0.0620      0.0000
			0.0800      0.0290      0.0000
			0.0390      0.0140      0.0000
			0.0190      0.0070      0.0000
			0.0090      0.0030      0.0000
			0.0040      0.0010      0.0000
			0.0020      0.0010      0.0000
			0.0010      0.0000      0.0000
			0.0000      0.0000      0.0000
			0.0000      0.0000      0.0000
			];
		cwf.C_2.uncompensated{1} = [
			  0.0000	  0.0000	  0.0000
			  0.0010	  0.0000	  0.0030
			  0.0040	  0.0000	  0.0170
			  0.0150	  0.0000	  0.0690
			  0.0740	  0.0020	  0.3500
			  0.2610	  0.0070	  1.2410
			  1.1700	  0.0320	  5.6050
			  3.0740	  0.1180	 14.9670
			  4.0660	  0.2590	 20.3460
			  3.9510	  0.4370	 20.7690
			  3.4210	  0.6840	 19.6240
			  2.2920	  1.0420	 15.1530
			  1.0660	  1.6000	  9.2940
			  0.3250	  2.3320	  5.1150
			  0.0250	  3.3750	  2.7880
			  0.0520	  4.8230	  1.4810
			  0.5350	  6.4680	  0.6690
			  1.4960	  7.9510	  0.3810
			  2.7660	  9.1930	  0.1870
			  4.2740	  9.8890	  0.0810
			  5.8910	  9.8980	  0.0360
			  7.3530	  9.1860	  0.0190
			  8.4590	  8.0080	  0.0150
			  9.0360	  6.6210	  0.0100
			  9.0050	  5.3020	  0.0070
			  8.3800	  4.1680	  0.0030
			  7.1110	  3.1470	  0.0010
			  5.3000	  2.1740	  0.0000
			  3.6690	  1.4270	  0.0000
			  2.3200	  0.8730	  0.0000
			  1.3330	  0.4920	  0.0000
			  0.6830	  0.2500	  0.0000
			  0.3560	  0.1290	  0.0000
			  0.1620	  0.0590	  0.0000
			  0.0770	  0.0280	  0.0000
			  0.0380	  0.0140	  0.0000
			  0.0180	  0.0060	  0.0000
			  0.0080	  0.0030	  0.0000
			  0.0040	  0.0010	  0.0000
			  0.0020	  0.0010	  0.0000
			  0.0010	  0.0000	  0.0000
			  0.0000	  0.0000	  0.0000
			  0.0000	  0.0000	  0.0000
			];
		cwf.C_2.uncompensated{2} = [
		   -0.0980    0.0010   -0.4660
			2.3730    0.0440   11.2960
			8.5950    0.4910   42.5610
			6.9390    1.3080   39.8990
			2.0450    3.0620   18.4510
		   -0.2170    6.5960    4.7280
			0.8810   12.9250    1.3410
			5.4060   18.6500    0.3190
		   11.8420   20.1430    0.0590
		   17.1690   16.0950    0.0280
		   18.3830   10.5370    0.0130
		   14.3480    6.2110    0.0020
			7.1480    2.7430         0
			2.4840    0.9110         0
			0.6000    0.2180         0
			0.1750    0.0630         0
			];

		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  C/10

		cwf.C_10.whitepoint = [97.285 100.000 116.145];
		cwf.C_10.wl = lam(1);
		cwf.C_10.compensated{1} = [
			0.0000      0.0000      0.0000
			0.0000      0.0000      0.0000
			0.0000      0.0000     -0.0020
			0.0060      0.0010      0.0250
			0.1020      0.0110      0.4570
			0.5940      0.0600      2.7280
			1.7050      0.1790      8.1170
			3.0250      0.3720     14.9330
			3.9440      0.6380     20.2290
			3.9190      0.9410     21.0680
			3.1780      1.3400     18.3610
			2.0470      1.9480     13.7680
			0.8560      2.6950      8.2180
			0.1710      3.5020      4.2730
			0.0400      4.3870      2.0880
			0.3250      5.2910      0.9860
			0.9700      6.2740      0.4930
			1.9710      7.3190      0.2520
			3.2710      8.3390      0.1170
			4.7550      8.8960      0.0350
			6.3190      8.9280      0.0010
			7.6370      8.3110      0.0000
			8.4640      7.2530      0.0000
			8.8550      6.1580      0.0000
			8.5890      5.0320      0.0000
			7.7470      3.9690      0.0000
			6.4270      2.9900      0.0000
			4.8370      2.1160      0.0000
			3.2400      1.3500      0.0000
			2.0110      0.8070      0.0000
			1.1430      0.4510      0.0000
			0.5970      0.2340      0.0000
			0.2920      0.1140      0.0000
			0.1360      0.0530      0.0000
			0.0620      0.0240      0.0000
			0.0280      0.0110      0.0000
			0.0130      0.0050      0.0000
			0.0060      0.0020      0.0000
			0.0030      0.0010      0.0000
			0.0010      0.0000      0.0000
			0.0010      0.0000      0.0000
			0.0000      0.0000      0.0000
			0.0000      0.0000      0.0000
			];
		cwf.C_10.uncompensated{1} = [
			  0.0000	  0.0000	  0.0000
			  0.0000	  0.0000	  0.0000
			  0.0000	  0.0000	  0.0000
			  0.0060	  0.0010	  0.0250
			  0.0710	  0.0070	  0.3170
			  0.5190	  0.0540	  2.3620
			  1.6900	  0.1730	  7.9950
			  3.0500	  0.3640	 15.0150
			  4.0550	  0.6380	 20.7510
			  3.9740	  0.9360	 21.3640
			  3.2070	  1.3160	 18.4570
			  2.0670	  1.9380	 13.9570
			  0.7920	  2.6930	  7.9680
			  0.1230	  3.4890	  4.1260
			  0.0080	  4.3950	  2.0060
			  0.2970	  5.2760	  0.9350
			  0.9390	  6.2750	  0.4800
			  1.9440	  7.2990	  0.2440
			  3.2590	  8.4010	  0.1140
			  4.7390	  8.9260	  0.0300
			  6.3400	  8.9950	 -0.0030
			  7.6940	  8.3570	  0.0010
			  8.4790	  7.2360	  0.0000
			  8.9290	  6.1710	  0.0000
			  8.6300	  5.0200	  0.0000
			  7.7940	  3.9660	  0.0000
			  6.4460	  2.9780	  0.0000
			  4.8480	  2.1140	  0.0000
			  3.1910	  1.3230	  0.0000
			  1.9860	  0.7930	  0.0000
			  1.1140	  0.4390	  0.0000
			  0.5770	  0.2260	  0.0000
			  0.2800	  0.1090	  0.0000
			  0.1300	  0.0500	  0.0000
			  0.0590	  0.0230	  0.0000
			  0.0270	  0.0100	  0.0000
			  0.0120	  0.0050	  0.0000
			  0.0050	  0.0020	  0.0000
			  0.0030	  0.0010	  0.0000
			  0.0010	  0.0000	  0.0000
			  0.0010	  0.0000	  0.0000
			  0.0000	  0.0000	  0.0000
			  0.0000	  0.0000	  0.0000
			];

		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  D50/2

		cwf.D50_2.whitepoint = [96.422 100.000 82.521];
		cwf.D50_2.wl = lam(1);
		cwf.D50_2.compensated{1} = [
			0.0000      0.0000      0.0010
			0.0010      0.0000      0.0050
			0.0030      0.0000      0.0130
			0.0120      0.0000      0.0570
			0.0600      0.0020      0.2850
			0.2340      0.0060      1.1130
			0.7750      0.0230      3.7230
			1.6100      0.0660      7.8620
			2.4530      0.1620     12.3090
			2.7770      0.3130     14.6470
			2.5000      0.5140     14.3460
			1.7170      0.7980     11.2990
			0.8610      1.2390      7.3090
			0.2830      1.8390      4.1280
			0.0400      2.9480      2.4660
			0.0880      4.6320      1.4470
			0.5930      6.5870      0.7360
			1.5900      8.3080      0.4010
			2.7990      9.1970      0.1960
			4.2070      9.6500      0.0850
			5.6570      9.4710      0.0370
			7.1320      8.9020      0.0200
			8.5400      8.1120      0.0150
			9.2550      6.8290      0.0100
			9.8350      5.8380      0.0070
			9.4690      4.7530      0.0040
			8.0090      3.5730      0.0020
			5.9260      2.4430      0.0010
			4.1710      1.6290      0.0000
			2.6090      0.9840      0.0000
			1.5410      0.5700      0.0000
			0.8550      0.3130      0.0000
			0.4340      0.1580      0.0000
			0.1940      0.0700      0.0000
			0.0970      0.0350      0.0000
			0.0500      0.0180      0.0000
			0.0220      0.0080      0.0000
			0.0120      0.0040      0.0000
			0.0060      0.0020      0.0000
			0.0020      0.0010      0.0000
			0.0010      0.0000      0.0000
			0.0010      0.0000      0.0000
			0.0000      0.0000      0.0000
			];
		cwf.D50_2.uncompensated{1} = [
			  0.0000	  0.0000	  0.0000
			  0.0010	  0.0000	  0.0050
			  0.0030	  0.0000	  0.0140
			  0.0080	  0.0000	  0.0390
			  0.0580	  0.0020	  0.2770
			  0.1910	  0.0050	  0.9060
			  0.7510	  0.0210	  3.6030
			  1.5920	  0.0600	  7.7470
			  2.5190	  0.1580	 12.5930
			  2.8240	  0.3100	 14.8340
			  2.5560	  0.5110	 14.6590
			  1.7170	  0.7760	 11.3440
			  0.8320	  1.2460	  7.2400
			  0.2500	  1.7830	  3.9340
			  0.0250	  2.8920	  2.4470
			  0.0470	  4.6100	  1.4320
			  0.5380	  6.5860	  0.6880
			  1.5900	  8.4350	  0.4030
			  2.7700	  9.1850	  0.1860
			  4.2100	  9.7330	  0.0800
			  5.6620	  9.5030	  0.0350
			  7.0920	  8.8820	  0.0190
			  8.6810	  8.2250	  0.0160
			  9.1750	  6.7280	  0.0100
			  9.9660	  5.8840	  0.0080
			  9.5560	  4.7520	  0.0030
			  8.0990	  3.5840	  0.0020
			  5.8350	  2.3920	  0.0000
			  4.1990	  1.6330	  0.0000
			  2.5390	  0.9540	  0.0000
			  1.5170	  0.5600	  0.0000
			  0.8310	  0.3040	  0.0000
			  0.4230	  0.1530	  0.0000
			  0.1780	  0.0640	  0.0000
			  0.0960	  0.0350	  0.0000
			  0.0490	  0.0180	  0.0000
			  0.0200	  0.0070	  0.0000
			  0.0120	  0.0040	  0.0000
			  0.0060	  0.0020	  0.0000
			  0.0020	  0.0010	  0.0000
			  0.0010	  0.0000	  0.0000
			  0.0010	  0.0000	  0.0000
			  0.0000	  0.0000	  0.0000
			];

		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  D50/10

		cwf.D50_10.whitepoint = [96.720 100.000 81.427];
		cwf.D50_10.wl = lam(1);
		cwf.D50_10.compensated{1} = [
			0.0000      0.0000      0.0000
			0.0000      0.0000      0.0000
			0.0000      0.0000     -0.0020
			0.0040      0.0000      0.0170
			0.0830      0.0090      0.3710
			0.4270      0.0440      1.9660
			1.0490      0.1100      4.9890
			1.6680      0.2040      8.2310
			2.4870      0.4030     12.7580
			2.8140      0.6770     15.1290
			2.4040      1.0120     13.8860
			1.5650      1.4900     10.5280
			0.6710      2.1080      6.4420
			0.1350      2.7790      3.3920
			0.0350      3.8500      1.8240
			0.3170      5.1430      0.9600
			1.0100      6.5130      0.5130
			2.0980      7.7910      0.2690
			3.3410      8.5250      0.1200
			4.7450      8.8770      0.0350
			6.1830      8.7420      0.0010
			7.5600      8.2220      0.0000
			8.7330      7.4850      0.0000
			9.2730      6.4490      0.0000
			9.5860      5.6130      0.0000
			8.9590      4.5920      0.0000
			7.4190      3.4520      0.0000
			5.4710      2.3920      0.0000
			3.7210      1.5500      0.0000
			2.2680      0.9100      0.0000
			1.3160      0.5190      0.0000
			0.7280      0.2850      0.0000
			0.3540      0.1380      0.0000
			0.1550      0.0600      0.0000
			0.0760      0.0290      0.0000
			0.0360      0.0140      0.0000
			0.0150      0.0060      0.0000
			0.0080      0.0030      0.0000
			0.0040      0.0020      0.0000
			0.0020      0.0010      0.0000
			0.0010      0.0000      0.0000
			0.0000      0.0000      0.0000
			0.0000      0.0000      0.0000
			];
		cwf.D50_10.uncompensated{1} = [
			  0.0000	  0.0000	  0.0000
			  0.0000	  0.0000	  0.0000
			  0.0010	  0.0000	  0.0020
			  0.0020	  0.0000	  0.0090
			  0.0590	  0.0060	  0.2630
			  0.3850	  0.0400	  1.7510
			  1.0870	  0.1120	  5.1540
			  1.5980	  0.1900	  7.8640
			  2.5560	  0.3980	 13.0660
			  2.8880	  0.6750	 15.5110
			  2.4370	  1.0000	 14.0230
			  1.5740	  1.4690	 10.6230
			  0.6300	  2.1300	  6.3120
			  0.0960	  2.7150	  3.2270
			  0.0060	  3.8420	  1.7960
			  0.2840	  5.1380	  0.9190
			  0.9650	  6.5000	  0.5010
			  2.1010	  7.8720	  0.2630
			  3.3170	  8.5320	  0.1140
			  4.7450	  8.9310	  0.0310
			  6.1940	  8.7800	 -0.0030
			  7.5470	  8.2140	  0.0010
			  8.8470	  7.5570	  0.0000
			  9.2180	  6.3750	  0.0000
			  9.7120	  5.6630	  0.0000
			  9.0350	  4.5970	  0.0000
			  7.4650	  3.4470	  0.0000
			  5.4260	  2.3660	  0.0000
			  3.7130	  1.5410	  0.0000
			  2.2080	  0.8820	  0.0000
			  1.2890	  0.5090	  0.0000
			  0.7140	  0.2790	  0.0000
			  0.3380	  0.1310	  0.0000
			  0.1440	  0.0560	  0.0000
			  0.0750	  0.0290	  0.0000
			  0.0350	  0.0140	  0.0000
			  0.0140	  0.0050	  0.0000
			  0.0080	  0.0030	  0.0000
			  0.0040	  0.0020	  0.0000
			  0.0020	  0.0010	  0.0000
			  0.0010	  0.0000	  0.0000
			  0.0000	  0.0000	  0.0000
			  0.0000	  0.0000	  0.0000
			];

		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  D55/2

		cwf.D55_2.whitepoint = [95.682 100.000 92.149];
		cwf.D55_2.wl = lam(1);
		cwf.D55_2.compensated{1} = [
			0.0000      0.0000      0.0010
			0.0010      0.0000      0.0060
			0.0040      0.0000      0.0170
			0.0150      0.0000      0.0730
			0.0740      0.0020      0.3530
			0.2840      0.0080      1.3500
			0.9240      0.0270      4.4400
			1.8860      0.0770      9.2080
			2.8050      0.1860     14.0760
			3.1190      0.3520     16.4470
			2.7690      0.5700     15.8930
			1.8770      0.8720     12.3530
			0.9290      1.3380      7.8910
			0.3010      1.9600      4.3990
			0.0420      3.1010      2.5930
			0.0920      4.8220      1.5060
			0.6100      6.7790      0.7580
			1.6220      8.4760      0.4090
			2.8350      9.3140      0.1990
			4.2310      9.7060      0.0850
			5.6540      9.4670      0.0370
			7.0890      8.8480      0.0200
			8.4310      8.0090      0.0150
			9.0440      6.6740      0.0100
			9.5030      5.6410      0.0070
			9.0700      4.5530      0.0030
			7.6160      3.3980      0.0020
			5.5930      2.3060      0.0000
			3.8970      1.5220      0.0000
			2.4200      0.9130      0.0000
			1.4160      0.5240      0.0000
			0.7790      0.2850      0.0000
			0.3940      0.1430      0.0000
			0.1760      0.0640      0.0000
			0.0880      0.0320      0.0000
			0.0460      0.0160      0.0000
			0.0200      0.0070      0.0000
			0.0110      0.0040      0.0000
			0.0050      0.0020      0.0000
			0.0020      0.0010      0.0000
			0.0010      0.0000      0.0000
			0.0010      0.0000      0.0000
			0.0000      0.0000      0.0000
			];
		cwf.D55_2.uncompensated{1} = [
			  0.0000	  0.0000	  0.0000
			  0.0010	  0.0000	  0.0060
			  0.0040	  0.0000	  0.0190
			  0.0110	  0.0000	  0.0510
			  0.0720	  0.0020	  0.3430
			  0.2320	  0.0060	  1.1050
			  0.8970	  0.0260	  4.3030
			  1.8720	  0.0710	  9.1130
			  2.8810	  0.1810	 14.4050
			  3.1690	  0.3480	 16.6480
			  2.8310	  0.5670	 16.2380
			  1.8740	  0.8490	 12.3880
			  0.8960	  1.3460	  7.8070
			  0.2660	  1.9020	  4.1870
			  0.0260	  3.0420	  2.5700
			  0.0500	  4.8060	  1.4900
			  0.5540	  6.7790	  0.7070
			  1.6240	  8.6050	  0.4110
			  2.8070	  9.3030	  0.1880
			  4.2360	  9.7890	  0.0800
			  5.6600	  9.4970	  0.0350
			  7.0520	  8.8290	  0.0180
			  8.5750	  8.1230	  0.0150
			  8.9680	  6.5740	  0.0100
			  9.6260	  5.6810	  0.0080
			  9.1510	  4.5500	  0.0030
			  7.6980	  3.4060	  0.0020
			  5.5080	  2.2580	  0.0000
			  3.9160	  1.5230	  0.0000
			  2.3560	  0.8850	  0.0000
			  1.3930	  0.5140	  0.0000
			  0.7570	  0.2770	  0.0000
			  0.3830	  0.1390	  0.0000
			  0.1620	  0.0590	  0.0000
			  0.0870	  0.0310	  0.0000
			  0.0450	  0.0160	  0.0000
			  0.0180	  0.0070	  0.0000
			  0.0110	  0.0040	  0.0000
			  0.0050	  0.0020	  0.0000
			  0.0020	  0.0010	  0.0000
			  0.0010	  0.0000	  0.0000
			  0.0010	  0.0000	  0.0000
			  0.0000	  0.0000	  0.0000
			];

		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  D55/10

		cwf.D55_10.whitepoint = [95.799 100.000 90.926];
		cwf.D55_10.wl = lam(1);
		cwf.D55_10.compensated{1} = [
			0.0000      0.0000      0.0000
			0.0000      0.0000      0.0000
			0.0000      0.0000     -0.0020
			0.0050      0.0010      0.0220
			0.1020      0.0110      0.4570
			0.5150      0.0530      2.3700
			1.2450      0.1300      5.9220
			1.9440      0.2380      9.5960
			2.8290      0.4590     14.5170
			3.1440      0.7570     16.9060
			2.6510      1.1160     15.3090
			1.7030      1.6210     11.4530
			0.7210      2.2650      6.9210
			0.1430      2.9470      3.5970
			0.0370      4.0290      1.9080
			0.3290      5.3290      0.9950
			1.0340      6.6710      0.5250
			2.1300      7.9100      0.2730
			3.3670      8.5920      0.1210
			4.7490      8.8850      0.0350
			6.1510      8.6960      0.0010
			7.4790      8.1330      0.0000
			8.5800      7.3550      0.0000
			9.0190      6.2720      0.0000
			9.2180      5.3980      0.0000
			8.5400      4.3770      0.0000
			7.0200      3.2670      0.0000
			5.1390      2.2470      0.0000
			3.4590      1.4410      0.0000
			2.0940      0.8400      0.0000
			1.2040      0.4750      0.0000
			0.6600      0.2580      0.0000
			0.3190      0.1240      0.0000
			0.1410      0.0550      0.0000
			0.0680      0.0270      0.0000
			0.0330      0.0130      0.0000
			0.0140      0.0050      0.0000
			0.0070      0.0030      0.0000
			0.0040      0.0010      0.0000
			0.0020      0.0010      0.0000
			0.0010      0.0000      0.0000
			0.0000      0.0000      0.0000
			0.0000      0.0000      0.0000
			];
		cwf.D55_10.uncompensated{1} = [
			  0.0000	  0.0000	  0.0000
			  0.0000	  0.0000	  0.0000
			  0.0010	  0.0000	  0.0030
			  0.0030	  0.0000	  0.0120
			  0.0730	  0.0080	  0.3260
			  0.4660	  0.0480	  2.1220
			  1.2910	  0.1330	  6.1200
			  1.8700	  0.2220	  9.2030
			  2.9100	  0.4540	 14.8750
			  3.2240	  0.7550	 17.3230
			  2.6860	  1.1040	 15.4580
			  1.7100	  1.5990	 11.5430
			  0.6750	  2.2890	  6.7730
			  0.1010	  2.8820	  3.4180
			  0.0070	  4.0210	  1.8760
			  0.2960	  5.3290	  0.9520
			  0.9890	  6.6570	  0.5130
			  2.1340	  7.9930	  0.2670
			  3.3450	  8.6000	  0.1150
			  4.7510	  8.9390	  0.0310
			  6.1620	  8.7320	 -0.0030
			  7.4680	  8.1260	  0.0010
			  8.6970	  7.4260	  0.0000
			  8.9660	  6.1990	  0.0000
			  9.3360	  5.4420	  0.0000
			  8.6100	  4.3800	  0.0000
			  7.0610	  3.2610	  0.0000
			  5.0970	  2.2220	  0.0000
			  3.4460	  1.4300	  0.0000
			  2.0390	  0.8140	  0.0000
			  1.1780	  0.4650	  0.0000
			  0.6470	  0.2530	  0.0000
			  0.3050	  0.1190	  0.0000
			  0.1310	  0.0510	  0.0000
			  0.0670	  0.0260	  0.0000
			  0.0320	  0.0120	  0.0000
			  0.0120	  0.0050	  0.0000
			  0.0070	  0.0030	  0.0000
			  0.0040	  0.0010	  0.0000
			  0.0010	  0.0010	  0.0000
			  0.0010	  0.0000	  0.0000
			  0.0000	  0.0000	  0.0000
			  0.0000	  0.0000	  0.0000
			];

		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  D65/2

		cwf.D65_2.whitepoint = [95.047 100.000 108.883];
		cwf.D65_2.wl = lam(1);
		cwf.D65_2.compensated{1} = [
			0.0000      0.0000      0.0010
			0.0020      0.0000      0.0100
			0.0060      0.0000      0.0260
			0.0220      0.0010      0.1040
			0.1010      0.0030      0.4770
			0.3760      0.0100      1.7880
			1.2000      0.0350      5.7650
			2.3960      0.0980     11.6980
			3.4180      0.2260     17.1500
			3.6990      0.4170     19.5060
			3.2270      0.6640     18.5200
			2.1490      0.9980     14.1370
			1.0420      1.5010      8.8500
			0.3330      2.1640      4.8560
			0.0450      3.3520      2.8020
			0.0980      5.1290      1.6020
			0.6370      7.0760      0.7910
			1.6670      8.7080      0.4200
			2.8840      9.4740      0.2020
			4.2500      9.7520      0.0860
			5.6260      9.4190      0.0370
			6.9880      8.7220      0.0190
			8.2140      7.8020      0.0140
			8.7300      6.4420      0.0100
			9.0150      5.3510      0.0070
			8.4920      4.2630      0.0030
			7.0500      3.1450      0.0010
			5.1240      2.1130      0.0000
			3.5160      1.3730      0.0000
			2.1670      0.8180      0.0000
			1.2520      0.4630      0.0000
			0.6780      0.2480      0.0000
			0.3410      0.1240      0.0000
			0.1530      0.0550      0.0000
			0.0760      0.0270      0.0000
			0.0400      0.0140      0.0000
			0.0180      0.0060      0.0000
			0.0090      0.0030      0.0000
			0.0050      0.0020      0.0000
			0.0020      0.0010      0.0000
			0.0010      0.0000      0.0000
			0.0000      0.0000      0.0000
			0.0000      0.0000      0.0000
			];
		cwf.D65_2.uncompensated{1} = [
			  0.0000	  0.0000	  0.0000
			  0.0020	  0.0000	  0.0090
			  0.0060	  0.0000	  0.0290
			  0.0160	  0.0000	  0.0770
			  0.0970	  0.0030	  0.4600
			  0.3110	  0.0090	  1.4770
			  1.1640	  0.0330	  5.5810
			  2.4000	  0.0920	 11.6840
			  3.5060	  0.2210	 17.5320
			  3.7550	  0.4130	 19.7290
			  3.2980	  0.6620	 18.9210
			  2.1410	  0.9730	 14.1610
			  1.0010	  1.5090	  8.7300
			  0.2930	  2.1070	  4.6230
			  0.0280	  3.2880	  2.7690
			  0.0540	  5.1220	  1.5840
			  0.5810	  7.0820	  0.7360
			  1.6680	  8.8330	  0.4210
			  2.8600	  9.4720	  0.1910
			  4.2570	  9.8300	  0.0810
			  5.6320	  9.4460	  0.0340
			  6.9600	  8.7090	  0.0180
			  8.3440	  7.9010	  0.0150
			  8.6760	  6.3570	  0.0090
			  9.1200	  5.3790	  0.0070
			  8.5680	  4.2590	  0.0030
			  7.1190	  3.1490	  0.0010
			  5.0490	  2.0700	  0.0000
			  3.5220	  1.3700	  0.0000
			  2.1120	  0.7940	  0.0000
			  1.2290	  0.4540	  0.0000
			  0.6580	  0.2400	  0.0000
			  0.3310	  0.1200	  0.0000
			  0.1420	  0.0510	  0.0000
			  0.0740	  0.0270	  0.0000
			  0.0390	  0.0140	  0.0000
			  0.0160	  0.0060	  0.0000
			  0.0090	  0.0030	  0.0000
			  0.0050	  0.0020	  0.0000
			  0.0020	  0.0010	  0.0000
			  0.0010	  0.0000	  0.0000
			  0.0010	  0.0000	  0.0000
			  0.0000	  0.0000	  0.0000
			];

		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  D65/10

		cwf.D65_10.whitepoint = [94.811 100.000 107.304];
		cwf.D65_10.wl = lam([1 2]);
		cwf.D65_10.compensated{1} = [
			0.0000      0.0000      0.0000
			0.0000      0.0000      0.0000
			0.0000      0.0000     -0.0020
			0.0080      0.0010      0.0330
			0.1370      0.0140      0.6120
			0.6760      0.0690      3.1100
			1.6030      0.1680      7.6270
			2.4510      0.3000     12.0950
			3.4180      0.5540     17.5370
			3.6990      0.8900     19.8880
			3.0640      1.2900     17.6950
			1.9330      1.8380     13.0000
			0.8020      2.5200      7.6990
			0.1560      3.2260      3.9380
			0.0390      4.3200      2.0460
			0.3470      5.6210      1.0490
			1.0700      6.9070      0.5440
			2.1700      8.0590      0.2780
			3.3970      8.6680      0.1220
			4.7320      8.8550      0.0350
			6.0700      8.5810      0.0010
			7.3110      7.9510      0.0000
			8.2910      7.1060      0.0000
			8.6340      6.0040      0.0000
			8.6720      5.0790      0.0000
			7.9300      4.0650      0.0000
			6.4460      2.9990      0.0000
			4.6690      2.0420      0.0000
			3.0950      1.2900      0.0000
			1.8590      0.7460      0.0000
			1.0560      0.4170      0.0000
			0.5700      0.2230      0.0000
			0.2740      0.1070      0.0000
			0.1210      0.0470      0.0000
			0.0580      0.0230      0.0000
			0.0280      0.0110      0.0000
			0.0120      0.0050      0.0000
			0.0060      0.0020      0.0000
			0.0030      0.0010      0.0000
			0.0010      0.0010      0.0000
			0.0010      0.0000      0.0000
			0.0000      0.0000      0.0000
			0.0000      0.0000      0.0000
			];
		cwf.D65_10.uncompensated{1} = [
			  0.0000	  0.0000	  0.0000
			  0.0000	  0.0000	 -0.0010
			  0.0010	  0.0000	  0.0040
			  0.0050	  0.0000	  0.0200
			  0.0970	  0.0100	  0.4360
			  0.6160	  0.0640	  2.8080
			  1.6600	  0.1710	  7.8680
			  2.3770	  0.2830	 11.7030
			  3.5120	  0.5490	 17.9580
			  3.7890	  0.8880	 20.3580
			  3.1030	  1.2770	 17.8610
			  1.9370	  1.8170	 13.0850
			  0.7470	  2.5450	  7.5100
			  0.1100	  3.1640	  3.7430
			  0.0070	  4.3090	  2.0030
			  0.3140	  5.6310	  1.0040
			  1.0270	  6.8960	  0.5290
			  2.1740	  8.1360	  0.2710
			  3.3800	  8.6840	  0.1160
			  4.7350	  8.9030	  0.0300
			  6.0810	  8.6140	 -0.0030
			  7.3100	  7.9500	  0.0010
			  8.3930	  7.1640	  0.0000
			  8.6030	  5.9450	  0.0000
			  8.7710	  5.1100	  0.0000
			  7.9960	  4.0670	  0.0000
			  6.4760	  2.9900	  0.0000
			  4.6350	  2.0200	  0.0000
			  3.0740	  1.2750	  0.0000
			  1.8140	  0.7240	  0.0000
			  1.0310	  0.4070	  0.0000
			  0.5570	  0.2180	  0.0000
			  0.2610	  0.1020	  0.0000
			  0.1140	  0.0440	  0.0000
			  0.0570	  0.0220	  0.0000
			  0.0280	  0.0110	  0.0000
			  0.0110	  0.0040	  0.0000
			  0.0060	  0.0020	  0.0000
			  0.0030	  0.0010	  0.0000
			  0.0010	  0.0000	  0.0000
			  0.0000	  0.0000	  0.0000
			  0.0000	  0.0000	  0.0000
			  0.0000	  0.0000	  0.0000
			];

		cwf.D65_10.uncompensated{2} = [
			0.0590    0.0120    0.2240
			2.9510    0.2800   13.7680
			7.2270    1.0420   36.8080
			6.5780    2.5340   37.8270
			1.2780    4.8720   14.2260
		   -0.2590    8.4380    3.2540
			1.9510   14.0300    1.0250
			6.7510   17.7150    0.1840
		   12.2230   17.4070   -0.0130
		   16.7790   14.2100    0.0040
		   17.7930   10.1210   -0.0010
		   13.1350    5.9710         0
			5.8590    2.3990         0
			1.9010    0.7410         0
			0.4690    0.1840         0
			0.1170    0.0450         0
			];

		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  D75/2
		cwf.D75_2.whitepoint = [94.972 100.000 122.638];
		cwf.D75_2.wl = lam(1);
		cwf.D75_2.compensated{1} = [
			0.0000      0.0000      0.0020
			0.0030      0.0000      0.0130
			0.0070      0.0000      0.0350
			0.0280      0.0010      0.1320
			0.1240      0.0030      0.5870
			0.4570      0.0120      2.1760
			1.4390      0.0430      6.9160
			2.8090      0.1150     13.7140
			3.9260      0.2600     19.7020
			4.1820      0.4720     22.0550
			3.6000      0.7410     20.6600
			2.3640      1.0980     15.5510
			1.1330      1.6320      9.6210
			0.3570      2.3210      5.2090
			0.0480      3.5510      2.9670
			0.1030      5.3650      1.6760
			0.6550      7.2810      0.8140
			1.6980      8.8730      0.4270
			2.9120      9.5670      0.2040
			4.2560      9.7660      0.0860
			5.5840      9.3500      0.0360
			6.8790      8.5860      0.0190
			8.0320      7.6290      0.0140
			8.4780      6.2560      0.0100
			8.6770      5.1510      0.0060
			8.1050      4.0680      0.0030
			6.6730      2.9770      0.0010
			4.8040      1.9810      0.0000
			3.2740      1.2790      0.0000
			2.0080      0.7570      0.0000
			1.1510      0.4260      0.0000
			0.6180      0.2260      0.0000
			0.3090      0.1120      0.0000
			0.1390      0.0500      0.0000
			0.0680      0.0250      0.0000
			0.0360      0.0130      0.0000
			0.0160      0.0060      0.0000
			0.0080      0.0030      0.0000
			0.0040      0.0020      0.0000
			0.0020      0.0010      0.0000
			0.0010      0.0000      0.0000
			0.0000      0.0000      0.0000
			0.0000      0.0000      0.0000
			];
		cwf.D75_2.uncompensated{1} = [
			  0.0000	  0.0000	  0.0000
			  0.0030	  0.0000	  0.0120
			  0.0080	  0.0000	  0.0380
			  0.0210	  0.0010	  0.0980
			  0.1200	  0.0030	  0.5670
			  0.3780	  0.0100	  1.7980
			  1.4030	  0.0400	  6.7280
			  2.8200	  0.1080	 13.7270
			  4.0280	  0.2540	 20.1460
			  4.2440	  0.4670	 22.3010
			  3.6770	  0.7390	 21.1060
			  2.3500	  1.0710	 15.5520
			  1.0870	  1.6420	  9.4850
			  0.3130	  2.2620	  4.9510
			  0.0290	  3.4840	  2.9290
			  0.0580	  5.3710	  1.6570
			  0.5990	  7.2810	  0.7540
			  1.7020	  9.0050	  0.4300
			  2.8900	  9.5640	  0.1920
			  4.2650	  9.8450	  0.0810
			  5.5920	  9.3750	  0.0340
			  6.8530	  8.5710	  0.0180
			  8.1610	  7.7250	  0.0150
			  8.4290	  6.1740	  0.0090
			  8.7770	  5.1760	  0.0070
			  8.1760	  4.0640	  0.0030
			  6.7370	  2.9800	  0.0010
			  4.7280	  1.9380	  0.0000
			  3.2790	  1.2750	  0.0000
			  1.9560	  0.7350	  0.0000
			  1.1280	  0.4170	  0.0000
			  0.5990	  0.2190	  0.0000
			  0.3010	  0.1090	  0.0000
			  0.1280	  0.0460	  0.0000
			  0.0670	  0.0240	  0.0000
			  0.0360	  0.0130	  0.0000
			  0.0140	  0.0050	  0.0000
			  0.0090	  0.0030	  0.0000
			  0.0040	  0.0020	  0.0000
			  0.0020	  0.0010	  0.0000
			  0.0010	  0.0000	  0.0000
			  0.0000	  0.0000	  0.0000
			  0.0000	  0.0000	  0.0000
			];

		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  D75/10

		cwf.D75_10.whitepoint = [94.416 100.000 120.641];
		cwf.D75_10.wl = lam(1);
		cwf.D75_10.compensated{1} = [
			0.0000      0.0000      0.0000
			0.0000      0.0000      0.0000
			0.0000      0.0000     -0.0020
			0.0100      0.0010      0.0420
			0.1670      0.0180      0.7490
			0.8160      0.0830      3.7550
			1.9110      0.2000      9.0910
			2.8550      0.3500     14.0890
			3.9000      0.6320     20.0110
			4.1550      1.0000     22.3410
			3.3960      1.4300     19.6120
			2.1120      2.0080     14.2050
			0.8660      2.7210      8.3160
			0.1670      3.4380      4.1970
			0.0410      4.5460      2.1510
			0.3600      5.8420      1.0900
			1.0940      7.0610      0.5560
			2.1970      8.1580      0.2810
			3.4080      8.6960      0.1220
			4.7080      8.8090      0.0340
			5.9850      8.4620      0.0010
			7.1500      7.7760      0.0000
			8.0550      6.9030      0.0000
			8.3290      5.7930      0.0000
			8.2930      4.8570      0.0000
			7.5190      3.8540      0.0000
			6.0600      2.8200      0.0000
			4.3490      1.9020      0.0000
			2.8640      1.1930      0.0000
			1.7110      0.6870      0.0000
			0.9640      0.3800      0.0000
			0.5160      0.2020      0.0000
			0.2470      0.0960      0.0000
			0.1090      0.0420      0.0000
			0.0520      0.0200      0.0000
			0.0260      0.0100      0.0000
			0.0110      0.0040      0.0000
			0.0060      0.0020      0.0000
			0.0030      0.0010      0.0000
			0.0010      0.0000      0.0000
			0.0000      0.0000      0.0000
			0.0000      0.0000      0.0000
			0.0000      0.0000      0.0000
			];
		cwf.D75_10.uncompensated{1} = [
			  0.0000	  0.0000	  0.0000
			  0.0000	  0.0000	 -0.0010
			  0.0010	  0.0000	  0.0050
			  0.0060	  0.0010	  0.0260
			  0.1190	  0.0130	  0.5350
			  0.7450	  0.0770	  3.3960
			  1.9850	  0.2050	  9.4100
			  2.7730	  0.3300	 13.6520
			  4.0090	  0.6280	 20.5030
			  4.2540	  0.9980	 22.8590
			  3.4370	  1.4170	 19.7900
			  2.1120	  1.9860	 14.2750
			  0.8050	  2.7510	  8.1040
			  0.1160	  3.3740	  3.9810
			  0.0080	  4.5340	  2.1050
			  0.3280	  5.8630	  1.0430
			  1.0510	  7.0420	  0.5390
			  2.2030	  8.2410	  0.2740
			  3.3920	  8.7110	  0.1160
			  4.7130	  8.8580	  0.0300
			  5.9970	  8.4930	 -0.0030
			  7.1490	  7.7730	  0.0010
			  8.1540	  6.9590	  0.0000
			  8.3030	  5.7360	  0.0000
			  8.3860	  4.8850	  0.0000
			  7.5800	  3.8550	  0.0000
			  6.0880	  2.8110	  0.0000
			  4.3120	  1.8790	  0.0000
			  2.8430	  1.1790	  0.0000
			  1.6690	  0.6660	  0.0000
			  0.9400	  0.3710	  0.0000
			  0.5040	  0.1970	  0.0000
			  0.2360	  0.0920	  0.0000
			  0.1020	  0.0400	  0.0000
			  0.0510	  0.0200	  0.0000
			  0.0250	  0.0100	  0.0000
			  0.0100	  0.0040	  0.0000
			  0.0060	  0.0020	  0.0000
			  0.0030	  0.0010	  0.0000
			  0.0010	  0.0000	  0.0000
			  0.0000	  0.0000	  0.0000
			  0.0000	  0.0000	  0.0000
			  0.0000	  0.0000	  0.0000
			];

		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  F2/2

		cwf.F2_2.whitepoint = [99.186 100.000 67.393];
		cwf.F2_2.wl = lam(1);
		cwf.F2_2.compensated{1} = [
			 0.0000      0.0000      0.0000
			 0.0000      0.0000      0.0000
			 0.0010      0.0000      0.0030
			-0.0010      0.0000     -0.0060
			 0.0820      0.0020      0.3910
			 0.1690      0.0050      0.8020
			 0.1730      0.0010      0.8060
			 2.8600      0.1360     14.0650
			 3.9310      0.2340     19.5880
			 1.3380      0.1620      7.1140
			 1.4210      0.2940      8.1610
			 1.0110      0.4700      6.6520
			 0.5020      0.7230      4.2570
			 0.1660      1.0780      2.4180
			 0.0220      1.6140      1.3560
			 0.0450      2.4250      0.7570
			 0.3100      3.4660      0.3870
			 0.7930      4.4240      0.2230
			 2.9350      9.1370      0.1750
			 5.3050     12.5330      0.1220
			 6.4280     10.6760      0.0390
			10.0890     12.5200      0.0280
			13.5080     12.8720      0.0240
			13.0820      9.6550      0.0150
			11.9890      7.1250      0.0090
			 9.4530      4.7460      0.0040
			 6.3930      2.8500      0.0010
			 3.7110      1.5290      0.0000
			 1.9290      0.7530      0.0000
			 0.9060      0.3410      0.0000
			 0.3870      0.1430      0.0000
			 0.1520      0.0550      0.0000
			 0.0590      0.0210      0.0000
			 0.0230      0.0080      0.0000
			 0.0080      0.0030      0.0000
			 0.0030      0.0010      0.0000
			 0.0010      0.0000      0.0000
			 0.0010      0.0000      0.0000
			 0.0000      0.0000      0.0000
			 0.0000      0.0000      0.0000
			 0.0000      0.0000      0.0000
			 0.0000      0.0000      0.0000
			 0.0000      0.0000      0.0000
			];
		cwf.F2_2.uncompensated{1} = [
			  0.0000	  0.0000	  0.0000
			  0.0000	  0.0000	  0.0000
			  0.0010	  0.0000	  0.0040
			 -0.0070	  0.0000	 -0.0380
			  0.0820	  0.0020	  0.3900
			  0.1750	  0.0050	  0.8360
			 -0.0480	 -0.0100	 -0.2930
			  2.9940	  0.1390	 14.7070
			  4.2350	  0.2480	 21.0810
			  1.1150	  0.1450	  5.9920
			  1.4620	  0.2900	  8.3730
			  1.0200	  0.4630	  6.7270
			  0.4870	  0.7140	  4.2110
			  0.1500	  1.0630	  2.3530
			  0.0080	  1.5920	  1.3180
			  0.0250	  2.4060	  0.7380
			  0.2920	  3.4730	  0.3700
			  0.6560	  4.1120	  0.2140
			  2.9170	  9.2470	  0.1760
			  5.4090	 12.9680	  0.1240
			  6.2170	 10.3690	  0.0340
			 10.1090	 12.6440	  0.0270
			 13.8260	 13.1670	  0.0240
			 13.1360	  9.5980	  0.0140
			 12.1100	  7.1130	  0.0090
			  9.4970	  4.7060	  0.0030
			  6.3610	  2.8020	  0.0010
			  3.6370	  1.4840	  0.0000
			  1.8670	  0.7230	  0.0000
			  0.8640	  0.3240	  0.0000
			  0.3630	  0.1340	  0.0000
			  0.1400	  0.0510	  0.0000
			  0.0540	  0.0200	  0.0000
			  0.0210	  0.0080	  0.0000
			  0.0080	  0.0030	  0.0000
			  0.0030	  0.0010	  0.0000
			  0.0010	  0.0000	  0.0000
			  0.0010	  0.0000	  0.0000
			  0.0000	  0.0000	  0.0000
			  0.0000	  0.0000	  0.0000
			  0.0000	  0.0000	  0.0000
			  0.0000	  0.0000	  0.0000
			  0.0000	  0.0000	  0.0000
			];


		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  F2/10

		cwf.F2_10.whitepoint = [103.279 100.000 69.027];
		cwf.F2_10.wl = lam(1);
		cwf.F2_10.compensated{1} = [
			 0.0000      0.0000      0.0000
			 0.0000      0.0000      0.0000
			 0.0000      0.0000     -0.0010
			-0.0090     -0.0010     -0.0410
			 0.1330      0.0140      0.6030
			 0.3110      0.0320      1.4250
			 0.3100      0.0250      1.4180
			 2.9770      0.3950     14.8610
			 4.0740      0.6170     20.7110
			 1.3930      0.3540      7.5530
			 1.4020      0.5930      8.1030
			 0.9460      0.9000      6.3630
			 0.4010      1.2610      3.8520
			 0.0810      1.6710      2.0390
			 0.0190      2.1650      1.0300
			 0.1690      2.7640      0.5150
			 0.5430      3.5170      0.2770
			 1.0930      4.2620      0.1540
			 3.5620      8.6850      0.1070
			 6.1660     11.8380      0.0550
			 7.2090     10.1170     -0.0010
			10.9670     11.8670      0.0000
			14.1820     12.1910      0.0000
			13.4530      9.3570      0.0000
			11.9970      7.0320      0.0000
			 9.1830      4.7070      0.0000
			 6.0750      2.8250      0.0000
			 3.5170      1.5370      0.0000
			 1.7670      0.7360      0.0000
			 0.8080      0.3240      0.0000
			 0.3390      0.1340      0.0000
			 0.1330      0.0520      0.0000
			 0.0490      0.0190      0.0000
			 0.0190      0.0070      0.0000
			 0.0070      0.0030      0.0000
			 0.0030      0.0010      0.0000
			 0.0010      0.0000      0.0000
			 0.0000      0.0000      0.0000
			 0.0000      0.0000      0.0000
			 0.0000      0.0000      0.0000
			 0.0000      0.0000      0.0000
			 0.0000      0.0000      0.0000
			 0.0000      0.0000      0.0000
			];
		cwf.F2_10.uncompensated{1} = [
			  0.0000	  0.0000	  0.0000
			  0.0000	  0.0000	  0.0000
			  0.0010	  0.0000	  0.0030
			 -0.0200	 -0.0010	 -0.0970
			  0.1300	  0.0140	  0.5880
			  0.3260	  0.0340	  1.4940
			  0.0880	 -0.0050	  0.3030
			  3.1070	  0.4070	 15.4910
			  4.3870	  0.6580	 22.2880
			  1.1690	  0.3120	  6.4150
			  1.4410	  0.5870	  8.2940
			  0.9540	  0.8950	  6.4280
			  0.3830	  1.2570	  3.7940
			  0.0600	  1.6640	  1.9730
			  0.0020	  2.1560	  0.9890
			  0.1510	  2.7520	  0.4920
			  0.5280	  3.5190	  0.2670
			  0.9340	  3.9560	  0.1480
			  3.5510	  8.7910	  0.1080
			  6.2950	 12.2430	  0.0560
			  6.9840	  9.8280	 -0.0040
			 11.0120	 11.9850	  0.0000
			 14.5080	 12.4510	  0.0000
			 13.5120	  9.3150	  0.0000
			 12.1110	  7.0320	  0.0000
			  9.2080	  4.6710	  0.0000
			  6.0300	  2.7760	  0.0000
			  3.4500	  1.4960	  0.0000
			  1.7020	  0.7040	  0.0000
			  0.7670	  0.3050	  0.0000
			  0.3170	  0.1250	  0.0000
			  0.1220	  0.0480	  0.0000
			  0.0450	  0.0170	  0.0000
			  0.0170	  0.0070	  0.0000
			  0.0060	  0.0020	  0.0000
			  0.0020	  0.0010	  0.0000
			  0.0010	  0.0000	  0.0000
			  0.0000	  0.0000	  0.0000
			  0.0000	  0.0000	  0.0000
			  0.0000	  0.0000	  0.0000
			  0.0000	  0.0000	  0.0000
			  0.0000	  0.0000	  0.0000
			  0.0000	  0.0000	  0.0000
			];

		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  F7/2

		cwf.F7_2.whitepoint = [95.041 100.000 108.747];
		cwf.F7_2.wl = lam(1);
		cwf.F7_2.compensated{1} = [
			0.0000      0.0000      0.0000
			0.0000      0.0000     -0.0010
			0.0010      0.0000      0.0070
			0.0040      0.0000      0.0190
			0.1100      0.0030      0.5210
			0.2690      0.0070      1.2820
			0.4750      0.0090      2.2490
			3.9510      0.1830     19.4080
			5.4660      0.3310     27.2690
			2.5470      0.3000     13.5010
			2.5850      0.5340     14.8460
			1.8400      0.8540     12.1030
			0.9150      1.3180      7.7640
			0.3020      1.9640      4.4050
			0.0410      2.9790      2.4990
			0.0870      4.5070      1.4040
			0.5560      6.1770      0.6910
			1.2580      6.9240      0.3470
			3.6440     11.3270      0.2170
			5.5220     13.1460      0.1300
			4.9320      8.1670      0.0290
			7.1450      8.8390      0.0190
			9.6100      9.1760      0.0170
			8.8880      6.5530      0.0100
			8.8280      5.2410      0.0070
			7.9510      3.9910      0.0030
			6.4850      2.8920      0.0010
			4.7210      1.9470      0.0000
			3.1060      1.2130      0.0000
			1.9490      0.7350      0.0000
			1.0930      0.4040      0.0000
			0.4490      0.1640      0.0000
			0.1810      0.0660      0.0000
			0.0780      0.0280      0.0000
			0.0320      0.0110      0.0000
			0.0130      0.0050      0.0000
			0.0050      0.0020      0.0000
			0.0020      0.0010      0.0000
			0.0010      0.0000      0.0000
			0.0000      0.0000      0.0000
			0.0000      0.0000      0.0000
			0.0000      0.0000      0.0000
			0.0000      0.0000      0.0000
			];
		cwf.F7_2.uncompensated{1} = [
			  0.0000	  0.0000	  0.0000
			  0.0000	  0.0000	 -0.0010
			  0.0010	  0.0000	  0.0050
			 -0.0040	  0.0000	 -0.0210
			  0.1050	  0.0030	  0.4990
			  0.2660	  0.0070	  1.2650
			  0.2030	 -0.0050	  0.9040
			  4.1130	  0.1860	 20.1790
			  5.8340	  0.3460	 29.0650
			  2.3010	  0.2780	 12.2460
			  2.6500	  0.5270	 15.1850
			  1.8550	  0.8420	 12.2350
			  0.8890	  1.3030	  7.6840
			  0.2730	  1.9330	  4.2850
			  0.0160	  2.9370	  2.4320
			  0.0520	  4.4950	  1.3720
			  0.5370	  6.2540	  0.6610
			  1.1180	  6.6200	  0.3290
			  3.6860	 11.5410	  0.2140
			  5.7270	 13.7110	  0.1310
			  4.6990	  7.6980	  0.0210
			  7.1240	  8.8670	  0.0190
			  9.8750	  9.4220	  0.0170
			  8.8330	  6.4450	  0.0100
			  8.8950	  5.2360	  0.0070
			  7.9990	  3.9780	  0.0030
			  6.5100	  2.8790	  0.0010
			  4.7090	  1.9290	  0.0000
			  3.0680	  1.1920	  0.0000
			  1.9240	  0.7230	  0.0000
			  1.0760	  0.3970	  0.0000
			  0.4170	  0.1520	  0.0000
			  0.1680	  0.0610	  0.0000
			  0.0730	  0.0260	  0.0000
			  0.0290	  0.0110	  0.0000
			  0.0130	  0.0050	  0.0000
			  0.0050	  0.0020	  0.0000
			  0.0020	  0.0010	  0.0000
			  0.0010	  0.0000	  0.0000
			  0.0000	  0.0000	  0.0000
			  0.0000	  0.0000	  0.0000
			  0.0000	  0.0000	  0.0000
			  0.0000	  0.0000	  0.0000
			];

		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  F7/10

		cwf.F7_10.whitepoint = [95.792 100.000 107.686];
		cwf.F7_10.wl = lam(1);
		cwf.F7_10.compensated{1} = [
			 0.0000      0.0000      0.0000
			 0.0000      0.0000      0.0000
			 0.0000      0.0000     -0.0010
			-0.0070     -0.0010     -0.0340
			 0.1680      0.0170      0.7570
			 0.4860      0.0500      2.2290
			 0.7150      0.0670      3.3410
			 4.0000      0.5240     19.9330
			 5.4960      0.8420     27.9810
			 2.5690      0.6390     13.8890
			 2.4730      1.0460     14.2920
			 1.6690      1.5870     11.2240
			 0.7090      2.2300      6.8100
			 0.1440      2.9510      3.6030
			 0.0350      3.8730      1.8400
			 0.3080      4.9790      0.9270
			 0.9430      6.0800      0.4790
			 1.6740      6.4660      0.2320
			 4.2860     10.4380      0.1290
			 6.2290     12.0410      0.0590
			 5.3600      7.5010     -0.0020
			 7.5280      8.1220      0.0000
			 9.7830      8.4240      0.0000
			 8.8610      6.1580      0.0000
			 8.5630      5.0150      0.0000
			 7.4860      3.8370      0.0000
			 5.9770      2.7800      0.0000
			 4.3370      1.8970      0.0000
			 2.7570      1.1490      0.0000
			 1.6850      0.6760      0.0000
			 0.9290      0.3670      0.0000
			 0.3800      0.1490      0.0000
			 0.1470      0.0570      0.0000
			 0.0620      0.0240      0.0000
			 0.0250      0.0100      0.0000
			 0.0100      0.0040      0.0000
			 0.0040      0.0010      0.0000
			 0.0010      0.0010      0.0000
			 0.0010      0.0000      0.0000
			 0.0000      0.0000      0.0000
			 0.0000      0.0000      0.0000
			 0.0000      0.0000      0.0000
			 0.0000      0.0000      0.0000
			];
		cwf.F7_10.uncompensated{1} = [
			  0.0000	  0.0000	  0.0000
			  0.0000	  0.0000	  0.0000
			  0.0000	  0.0000	  0.0010
			 -0.0210	 -0.0010	 -0.1010
			  0.1560	  0.0160	  0.7000
			  0.4930	  0.0510	  2.2590
			  0.4610	  0.0300	  2.0550
			  4.1480	  0.5350	 20.6420
			  5.8630	  0.8850	 29.8190
			  2.3340	  0.5890	 12.6850
			  2.5320	  1.0340	 14.5810
			  1.6820	  1.5780	 11.3350
			  0.6770	  2.2240	  6.7110
			  0.1060	  2.9350	  3.4830
			  0.0040	  3.8580	  1.7700
			  0.2780	  4.9790	  0.8880
			  0.9340	  6.1390	  0.4620
			  1.5180	  6.1680	  0.2210
			  4.3420	 10.6340	  0.1260
			  6.4620	 12.5510	  0.0580
			  5.1080	  7.0730	 -0.0060
			  7.5200	  8.1490	  0.0000
			 10.0480	  8.6370	  0.0000
			  8.8090	  6.0650	  0.0000
			  8.6270	  5.0180	  0.0000
			  7.5210	  3.8260	  0.0000
			  5.9880	  2.7660	  0.0000
			  4.3320	  1.8860	  0.0000
			  2.7150	  1.1260	  0.0000
			  1.6590	  0.6630	  0.0000
			  0.9120	  0.3590	  0.0000
			  0.3540	  0.1380	  0.0000
			  0.1340	  0.0520	  0.0000
			  0.0580	  0.0220	  0.0000
			  0.0230	  0.0090	  0.0000
			  0.0090	  0.0030	  0.0000
			  0.0030	  0.0010	  0.0000
			  0.0010	  0.0010	  0.0000
			  0.0010	  0.0000	  0.0000
			  0.0000	  0.0000	  0.0000
			  0.0000	  0.0000	  0.0000
			  0.0000	  0.0000	  0.0000
			  0.0000	  0.0000	  0.0000
			];

		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  F11/2

		cwf.F11_2.whitepoint = [100.962 100.000 64.350];
		cwf.F11_2.wl = lam(1);
		cwf.F11_2.compensated{1} = [
			 0.0000      0.0000      0.0000
			 0.0000      0.0000      0.0000
			 0.0010      0.0000      0.0020
			-0.0050      0.0000     -0.0220
			 0.0590      0.0020      0.2810
			 0.0970      0.0030      0.4630
			 0.0240     -0.0040      0.0870
			 2.6870      0.1280     13.2070
			 3.9520      0.2370     19.7050
			 1.4710      0.1770      7.8190
			 1.3280      0.2740      7.6210
			 0.7230      0.2950      4.6850
			 0.4480      0.8030      4.0440
			 0.3260      1.9050      4.4580
			 0.0200      1.1040      1.0050
			 0.0060      0.4990      0.1210
			-0.0120      0.2440      0.0380
			-0.1550      0.1630      0.0370
			 8.9830     26.9550      0.4830
			10.5200     26.0540      0.2910
			 0.9930      1.3480     -0.0070
			 1.0640      1.2830      0.0020
			 6.7170      6.1910      0.0110
			 8.6970      6.5900      0.0100
			 6.1880      3.6690      0.0050
			27.0720     13.4150      0.0090
			13.8470      6.3290      0.0030
			 4.0030      1.6140      0.0000
			 0.8640      0.3350      0.0000
			 0.5410      0.2030      0.0000
			 0.3010      0.1110      0.0000
			 0.0960      0.0350      0.0000
			 0.0460      0.0170      0.0000
			 0.0280      0.0100      0.0000
			 0.0140      0.0050      0.0000
			 0.0180      0.0070      0.0000
			 0.0020      0.0010      0.0000
			 0.0000      0.0000      0.0000
			 0.0000      0.0000      0.0000
			 0.0000      0.0000      0.0000
			 0.0000      0.0000      0.0000
			 0.0000      0.0000      0.0000
			 0.0000      0.0000      0.0000
			];
		cwf.F11_2.uncompensated{1} = [
			  0.0000	  0.0000	  0.0000
			  0.0000	  0.0000	  0.0000
			  0.0010	  0.0000	  0.0050
			 -0.0090	  0.0000	 -0.0480
			  0.0610	  0.0020	  0.2910
			  0.1070	  0.0030	  0.5110
			 -0.2050	 -0.0140	 -1.0440
			  2.8000	  0.1300	 13.7580
			  4.2640	  0.2510	 21.2310
			  1.2770	  0.1640	  6.8490
			  1.3670	  0.2800	  7.8480
			  0.6950	  0.2550	  4.4950
			  0.4350	  0.7540	  3.9560
			  0.3410	  2.0630	  4.7780
			 -0.0040	  1.0880	  0.7920
			  0.0070	  0.4690	  0.0540
			 -0.0010	  0.2290	  0.0320
			 -0.9250	 -2.0670	  0.0000
			  9.6130	 29.2540	  0.5350
			 11.4380	 28.0300	  0.3000
			  0.1960	 -0.6950	 -0.0310
			  0.6020	  0.8700	  0.0020
			  7.0210	  6.5650	  0.0120
			  9.0700	  6.8660	  0.0110
			  4.2470	  2.6170	  0.0040
			 29.9030	 14.8120	  0.0100
			 13.5670	  6.1320	  0.0030
			  3.4460	  1.3290	  0.0000
			  0.6300	  0.2400	  0.0000
			  0.5340	  0.1990	  0.0000
			  0.2970	  0.1100	  0.0000
			  0.0840	  0.0310	  0.0000
			  0.0430	  0.0160	  0.0000
			  0.0280	  0.0100	  0.0000
			  0.0130	  0.0050	  0.0000
			  0.0200	  0.0070	  0.0000
			  0.0010	  0.0000	  0.0000
			  0.0000	  0.0000	  0.0000
			  0.0000	  0.0000	  0.0000
			  0.0000	  0.0000	  0.0000
			  0.0000	  0.0000	  0.0000
			  0.0000	  0.0000	  0.0000
			  0.0000	  0.0000	  0.0000
			];

		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  F11/10

		cwf.F11_10.whitepoint = [103.863 100.000 65.607];
		cwf.F11_10.wl = lam(1);
		cwf.F11_10.compensated{1} = [
			 0.0000      0.0000      0.0000
			 0.0000      0.0000      0.0000
			 0.0000      0.0000      0.0000
			-0.0100     -0.0010     -0.0440
			 0.0990      0.0100      0.4510
			 0.1820      0.0190      0.8290
			 0.0980      0.0030      0.4150
			 2.7960      0.3720     13.9640
			 4.1030      0.6250     20.8730
			 1.5340      0.3880      8.3100
			 1.3140      0.5540      7.5860
			 0.6810      0.5780      4.4980
			 0.3430      1.3800      3.6250
			 0.1760      2.9550      3.7890
			 0.0090      1.5060      0.7730
			 0.0340      0.5640      0.0740
			 0.0050      0.2570      0.0280
			-0.1450      0.1700      0.0270
			10.8520     25.6560      0.2930
			12.3200     24.6610      0.1480
			 1.0960      1.2740     -0.0100
			 1.1570      1.2140      0.0000
			 7.0360      5.8810      0.0000
			 8.9820      6.3820      0.0000
			 6.2040      3.6290      0.0000
			26.2640     13.3210      0.0000
			13.2280      6.2790      0.0000
			 3.7970      1.6310      0.0000
			 0.7940      0.3290      0.0000
			 0.4810      0.1920      0.0000
			 0.2640      0.1040      0.0000
			 0.0840      0.0330      0.0000
			 0.0380      0.0150      0.0000
			 0.0230      0.0090      0.0000
			 0.0110      0.0040      0.0000
			 0.0140      0.0050      0.0000
			 0.0020      0.0010      0.0000
			 0.0000      0.0000      0.0000
			 0.0000      0.0000      0.0000
			 0.0000      0.0000      0.0000
			 0.0000      0.0000      0.0000
			 0.0000      0.0000      0.0000
			 0.0000      0.0000      0.0000
			];
		cwf.F11_10.uncompensated{1} = [
			  0.0000	  0.0000	  0.0000
			  0.0000	  0.0000	  0.0000
			  0.0010	  0.0000	  0.0040
			 -0.0190	 -0.0010	 -0.0880
			  0.1020	  0.0110	  0.4600
			  0.1960	  0.0210	  0.8970
			 -0.1340	 -0.0280	 -0.7560
			  2.9080	  0.3810	 14.5020
			  4.4260	  0.6660	 22.4920
			  1.3390	  0.3550	  7.3270
			  1.3480	  0.5660	  7.7830
			  0.6570	  0.5130	  4.3130
			  0.3290	  1.3160	  3.5390
			  0.1760	  3.2060	  4.0530
			 -0.0060	  1.4640	  0.5810
			  0.0390	  0.5100	  0.0200
			  0.0150	  0.2380	  0.0240
			 -1.0700	 -1.9510	  0.0050
			 11.6430	 27.8540	  0.3270
			 13.3740	 26.5200	  0.1490
			  0.1590	 -0.6600	 -0.0230
			  0.6740	  0.8220	  0.0000
			  7.3620	  6.2260	  0.0000
			  9.3740	  6.6530	  0.0000
			  4.3090	  2.5970	  0.0000
			 29.0110	 14.7100	  0.0000
			 12.9300	  6.0800	  0.0000
			  3.2630	  1.3530	  0.0000
			  0.5710	  0.2320	  0.0000
			  0.4730	  0.1870	  0.0000
			  0.2610	  0.1030	  0.0000
			  0.0730	  0.0290	  0.0000
			  0.0360	  0.0140	  0.0000
			  0.0230	  0.0090	  0.0000
			  0.0100	  0.0040	  0.0000
			  0.0150	  0.0060	  0.0000
			  0.0010	  0.0000	  0.0000
			  0.0000	  0.0000	  0.0000
			  0.0000	  0.0000	  0.0000
			  0.0000	  0.0000	  0.0000
			  0.0000	  0.0000	  0.0000
			  0.0000	  0.0000	  0.0000
			  0.0000	  0.0000	  0.0000
			];
		end

% First two arguments (cwf and lam) added by Paul Centore on November 29, 2013, because Octave is automatically
% coercing the internal function getcwf into a subfunction.
	function z=getcwf(cwf, lam, name, varargin)
		z=[];
		if nargin==0
			z=cwf;
		else
			if nargin<1 || isempty(name); name=dcwf; end
			if ~ischar(name);error(illpar('Illuminant/Observer must be a char array'));end
			name=upper(name);
			if ~iscwf(name);error(illpar('Argument 1 is not a valid illuminant/observer'));end

			wanted=name;
			wanted(wanted=='/')='_';
			if isfield(cwf,wanted)
				if 3<=nargin
					comp=partialmatch(varargin{2},{'compensated','uncompensated'}, 'noerr');
					if isstruct(comp)
						error(illpar('Valid SpectrumType is ''compensated'' or ''uncompensated'''));
						end
				else
					comp=optgetpref('SpectrumType');
					end
				if 2<=nargin && ~isempty(varargin{1})
					wl=varargin{1};
				else
					wl=dwl;
					end
				if iswlrange(wl)
					wix=RangeIndex(lam,wl);
					if ~isempty(wix) && isfield(cwf.(wanted),comp)
						z.name=['ASTM ' name ' Table ' iff(strcmp('compensated',comp),'5','6')];
						z.whitepoint=cwf.(wanted).whitepoint;
						z.weights=cwf.(wanted).(comp){wix};
						z.wl=cwf.(wanted).wl{wix};
						end
				else
					error(illpar('Not a valid wavelength range'));
					end
				end
			end
		end

	z=[];
	error(nargchk(0,4,nargin));
	switch lower(op)
		case 'cwf'
% First two arguments (cwf and lam) added by Paul Centore on November 29, 2013, because Octave is automatically
% coercing the internal function getcwf into a subfunction.
			z=getcwf(cwf, lam, varargin{:});
		otherwise
			error(illpar('Unrecognized ASTM selector'));
		end
	end

function z=RangeIndex(lams,wl)
% Check that requested wavelength range snaps onto one of the wavelength
% ranges specified by lams. Return index if so, otherwise return [];
	z=[];
	delta=wl(2)-wl(1);
	for i=1:length(lams)
		lm=lams{i};
		dl=lm(2)-lm(1);
		if dl==delta && wl(1)<lm(end) && wl(end)>lm(1) && rem(wl(1),delta)==rem(lm(1),dl)
			z=i;
			break;
			end
		end

	end
